# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from funittest import logger
from funittest.utils import press_key_to_continue

try:
    from IPython.Shell import IPShellEmbed
except:
    IPShellEmbed = None
    
class BaseInterpreter(object):

    base=""
    _pause_on_ok = False
    _pause_on_ko = False
    _pause_on_interpreter_error = False

    def getBase(self):
        return self.base
    
    def setBase(self, base):
        self.base = base
    
    def annotate(self, annotation):
        # Used for annotating what the interpreter is supposed to do next
        logger.log_annotation(annotation)

    def pause_interpreter(self, prompt):
        from funittest import physical
        from funittest import dataprovider
        from funittest import logical
        from funittest import scenarios
        from funittest import tests
        from funittest import scripts
        from funittest import interpreter        
        if IPShellEmbed:
            try:
                __IPYTHON__
            except NameError:
                argv = ['']
                banner = exit_msg = ''
            else:
                # Command-line options for IPython (a list like sys.argv)
                argv = ['-pi1','In <\\#>:','-pi2','   .\\D.:','-po','Out<\\#>:']
                banner = '*** Nested interpreter ***'
                exit_msg = '*** Back in main IPython ***'    
            ipshell = IPShellEmbed(argv,banner=banner,exit_msg=exit_msg)
            ipshell() # this call anywhere in your program will start IPython
        else:
            press_key_to_continue(self, prompt)

    def pause_for_debugging(self, was_successful, exception_info=None):
        if was_successful:
            self.pause_on_ok()
        else:
            self.pause_on_ko()

    def pause_on_ok(self):
        if self.get_pause_on_ok():
            if self.is_dry_run():
                pass # Don't interrupt the flow in dry run mode
            else:
                self.pause_interpreter("OK")

    def pause_on_ko(self):            
        if self.get_pause_on_ko():
            if self.is_dry_run():
                pass # Don't interrupt the flow in dry run mode
            else:
                self.pause_interpreter("KO")

    def pause_on_interpreter_error(self):
        if self.get_pause_on_interpreter_error():
            if self.is_dry_run():
                pass # Don't interrupt the flow in dry run mode
            else:
                self.pause_interpreter("Error in interpreter. Press any key.")

    def get_pause_on_ok(self):
        return self._pause_on_ok

    def get_pause_on_ko(self):
        return self._pause_on_ko

    def get_pause_on_interpreter_error(self):
        return self._pause_on_interpreter_error

    def set_pause_on_ok(self, do_pause):
        if do_pause:
            self._pause_on_ok = True
        else:
            self._pause_on_ok = False

    def set_pause_on_ko(self, do_pause):
        if do_pause:
            self._pause_on_ko = True
        else:
            self._pause_on_ko = False

    def set_pause_on_interpreter_error(self, do_pause):
        if do_pause:
            self._pause_on_interpreter_error = True
        else:
            self._pause_on_interpreter_error = False
