from funittest.utils import _funittestExtensions
from funittest.utils import rescanEggs
from funittest.utils import ZopeApp
from pkg_resources import working_set
from pkg_resources import Requirement

import sys

def funittestExtensionsIds():
    """Return ids of funittest extensions.
    """
    return [e.key for e in _funittestExtensions()]

def scriptUsage():
    """Display usage and help about the script.
    """
    print "This script must be run by zopectl:"
    print "$ zopectl run %s profiles ..." % sys.argv[0]
    available = funittestExtensionsIds()
    if available:
        print "Available profiles: %s" % ','.join(available)
    sys.exit(1)

def configure_zope(zope_app):
    # The zope_app should have an app attribute containing the Zope app
    if not zope_app.app:
        scriptUsage()

    # Rescan for new eggs added in sys.path by this script
    rescanEggs()

    # Build a dict of all entry_points
    entry_points = dict()
    available = funittestExtensionsIds()
    
    for wanted in sys.argv[1:]:
        if not wanted in available:
            scriptUsage()

        egg = working_set.find(Requirement.parse(wanted))
        entry_points.update(egg.get_entry_map('funittest_zope'))

    # Start a session
    zope_app_session = zope_app.initSession('admin')

    # And run each entry points in alpha order
    order = entry_points.keys()
    order.sort()
    for name in order:
        configurator = entry_points[name].load()
        configurator(zope_app_session)
        zope_app.commit_transaction()

    # Close the session
    zope_app.closeSession(zope_app_session)

def main():
    """Run configuration script inside of Zope.
    """
    if len(sys.argv) < 2:
        scriptUsage()

    # First, get the Zope app in the locals var of the caller.
    zope_app = ZopeApp()

    # Now configure the Zope app using the entry points
    configure_zope(zope_app)
