#! /usr/bin/env python

import sys

from funittest import tests
from funittest import dataprovider
from funittest import scenarios
from funittest import logical
from funittest import physical
from funittest import scripts
from funittest import state

from funittest import load_product_version
from funittest.core.discovery import PresetDiscovery, ProductDiscovery
from funittest.utils import FileAsPageTemplate

from base import BaseCommand
from runner import run_command

models = dict(tests=tests,
              dataprovider=dataprovider,
              scenarios=scenarios,
              logical=logical,
              physical=physical,
              scripts=scripts,
              state=state)

class DocCommand(BaseCommand):
    """Generate documentation about available tests ands models.
    """

    description = "Generation documentation about available tests and models in HTML."

    def customOptions(self, parser):
        parser.add_option("-o",
                          "--output",
                          dest="output",
                          default='documentation.html',
                          help="Output file for the documentation (default: %default)")
        super(DocCommand, self).customOptions(parser)

    def run(self):
        """Do the work.
        """
        # Be sure to load all the products.
        for product in self.product_discovery.available():
            load_product_version(product)

        # Set options.
        options = dict(presets=self.preset_discovery,
                       products=self.product_discovery,)
        options.update(models)

        template = FileAsPageTemplate('templates/documentation.pt')
        template(options=options, filename=self.options.output)
        return 0

def main():
    """Entry point for setuptools.
    """
    sys.exit(run_command(DocCommand, simple=True))


if __name__ == '__main__':
    main()


