#! /usr/bin/env python

import sys

from funittest import dataprovider
from funittest import scenarios
from funittest import interpreter

from base import BaseCommand
from runner import run_command

from funittest.utils import normalizeClassName
from funittest.log import logger
from funittest.core.configuration import get_browser_config
from funittest.problems import SeleniumResponseError

class PresetCommand(BaseCommand):
    """Work with preset.
    """

    description = "Establishes funittest preconditions following the definitions in a preset configuration file."

    dataproviders = []

    def customOptions(self, parser):
        parser.add_option("-d", 
                          "--dataprovider", 
                          dest="dataprovider",
                          default=None,
                          help="Dataprovider to use")
        super(PresetCommand, self).customOptions(parser)

    def run(self):
        """Do the work.
        """
        if self.options.product:
            # Get the product from the dataprovider registry
            product = self.lookupProduct(dataprovider, 
                                         self.options.product)

            # Only execute the preset of a specific dataprovider?
            dataproviders = self.lookupModels(product, 
                                              self.options.dataprovider, 
                                              default=self.dataproviders)
        else:
            dataproviders = []

        browser_type, browser_url = get_browser_config()

        interpreter.setBrowserStartCommand("*" + browser_type)
        if browser_url:
            interpreter.setBrowserURL(browser_url)
        else:
            if interpreter.getBrowserURL():
                pass
            else:
                raise "Missing browser url"
        interpreter.start()
        try:
            interpreter.open('/')
        except SeleniumResponseError:
            interpreter.stop()
            return 0
         
        if self.options.preset:               
            login = self.preset_discovery.login(self.options.preset)
            login()

        established = 0
        for dp in dataproviders:
            # Only establish dataproviders whose.current variable is not an empty list
            if dp.establish_current:
                logger.start_establish_dataprovider(dp)
                try:
                    dp.establish(dp.current)
                except SeleniumResponseError:
                    logger.end_establish_dataprovider(dp)
                    logger.log_established_dataproviders(established)
                    interpreter.stop()
                    return 0
                logger.end_establish_dataprovider(dp)
                established = established + 1

        logger.log_established_dataproviders(established)

        interpreter.stop()
        return 0
        
def main():
    """Entry point for setuptools.
    """
    sys.exit(run_command(PresetCommand))

if __name__ == '__main__':
    main()

