#! /usr/bin/env python

import sys

from funittest import scenarios
from funittest.runner import scenariorunner

from base import ModelBasedCommand
from runner import run_command

class ScenarioCommand(ModelBasedCommand):
    """Command to run scenarios
    
    Supports choosing
    
    - a specific scenario to run
    
    - a specific user to login for the scenario
    """
    
    description = "Runs funittest scenarios and extensions, taking a preset configuration into account."

    def customOptions(self, parser):
        parser.add_option("-s", 
                          "--scenario",
                          dest="scenario",
                          help="The scenario to use")
        parser.add_option("-u",
                          "--user",
                          dest="user",
                          help="User to login for scenario")

        super(ScenarioCommand, self).customOptions(parser)


    def run(self):
        """Do the work.
        """
        
        product = self.lookupProduct(scenarios, self.options.product)
        models = self.lookupModels(product, self.options.scenario)

        if not self.displayDocumentation(product, models):
            for scenario in models:
                scr = scenariorunner.ScenarioConfigurationRunner
                runner = scr(scenario, self.options.extension,
                             self.vary, self.fix, self.options.user)
                configurations = runner.get_configurations()
                runner.run_configurations(configurations)
        return 0

def main():
    """Entry point for setuptools.
    """
    sys.exit(run_command(ScenarioCommand))

if __name__ == '__main__':
    main()

