#! /usr/bin/env python
import sys
from StringIO import StringIO

from funittest import tests
from funittest.runner.baserunner import TestRunner

from base import ModelBasedCommand
from runner import run_command
from funittest.log import logger

class TestCommand(ModelBasedCommand):
    """This script is used to run the functional tests. By default, all tests
       are run, but if you provide the test parameter, you can also run a 
       specific test you may be working on.
    """

    description = "Runs funittest tests."

    def customOptions(self, parser):
        """A specific test to be run can be defined on the command line.
        """
        parser.add_option("-t", 
                          "--test", 
                          dest="test",
                          help="The test to run")
        
        super(TestCommand, self).customOptions(parser)

    def run(self):
        """Functional tests are looked up in the test models of the product
           specified on the command line.
           If a specific test is chosen on the command line, only this is
           looked up.
           All test models are passed to the TestRunner, along with further
           information about any specific extension to use, or any parameters
           to vary or fix.
        """
        product = self.lookupProduct(tests, self.options.product)
        models = self.lookupModels(product, self.options.test)

        if not self.displayDocumentation(product, models):
            runner = TestRunner(models, self.options.extension, self.vary, self.fix)
            return runner.run()
        return 0

def main():
    """Entry point for setuptools. Setuptools don't care about the
    exit code, so set it here.
    """
    sys.exit(run_command(TestCommand))

if __name__ == '__main__':
    main()
