# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from zope.interface import implements

from funittest.utils import listAvailablePythonModules

from interfaces import IDiscovery
from configuration  import retrieveConfiguration

class BaseDiscovery(object):
    """Used to discover python modules in folders.
    """

    _search_path = 'funittest'
    _type = 'product'

    def __init__(self):
        self._modules = listAvailablePythonModules(self._search_path)
        self._defaults = {}
        config = retrieveConfiguration()
        if config:
            self._defaults = config.default()

    def available(self):
        """Search available modules, and return available one.
        """

        names = self._modules.keys()
        names.sort()
        return names

class PresetDiscovery(BaseDiscovery):
    """Used to discover available preset.
    """

    implements(IDiscovery)

    _search_path = 'funittest.preset'
    _type = 'preset'

    def preset(self, name):
        """Return the given preset.
        """
        module = self._modules[name]
        return __import__(module, globals(), locals(), ['preset']).preset

    def version(self, name):
        """Return the given preset version.
        """
        module = self._modules[name]
        return __import__(module, globals(), locals(), ['preset']).version

    def login(self, name):
        """Return the given preset login.
        """
        module = self._modules[name]
        return __import__(module, globals(), locals(), ['preset']).login


class ProductVersionDiscovery(BaseDiscovery):
    """Used to discover available product version.
    """

    _type = 'version'

    def __init__(self, product):
        self._search_path = 'funittest.lib.%s' % product
        super(ProductVersionDiscovery, self).__init__()

    def newest(self):
        return self.available()[-1]


class ProductDiscovery(BaseDiscovery):
    """Used to discover available product.
    """
    
    implements(IDiscovery)

    _search_path = 'funittest.lib'
    _type = 'product'
    
    def __init__(self):
        super(ProductDiscovery, self).__init__()
        self._versions = {}
        for product in self.available():
            self._versions[product] = ProductVersionDiscovery(product)

    def versions(self, product):
        return self._versions[product]


