from funittest.log import logger

class Interpreter(object):

    # The real interpreter type is selected at run-time, and wrapped here
    wrappedinterpreter = None

    def __init__(self, host="", port="", browserStartCommand="", browserURL=""):
        self._host=host
        self._port = port
        self._browserStartCommand = browserStartCommand
        self._browserURL = browserURL

    # Make it possible to call 
    # interpreter("localhost", 4444, "*firefox", "http://localhost:3100")
    # to get new interpreter settings
    def __call__(self, host, port, browserStartCommand, browserURL):
        self.__init__(host, port, browserStartCommand, browserURL)
        return self
    
    def __getattr__(self, attr):
        if self.wrappedinterpreter is None:
            from funittest.seleniumrc import SeleniumRC
            self.wrappedinterpreter = SeleniumRC(interpreter._host, interpreter._port, interpreter._browserStartCommand, interpreter._browserURL)
        if not hasattr(self.wrappedinterpreter, '_testcase'):
            from unittest import TestCase
            TestCase.runTest=lambda x:True
            self.wrappedinterpreter._testcase = TestCase()

        def wrapper(*args, **kw):
            # Try to get the method the normal way
            interpretermethod = getattr(self.wrappedinterpreter, attr)
            logger.start_interpreter_method(attr, args, kw)
            # We call the method with all arguments and keywords
            result = interpretermethod(*args, **kw)
            logger.end_interpreter_method(attr, args, kw)
            return result

        if attr.startswith("__") and attr.endswith("__"):
            return getattr(self.wrappedinterpreter, attr)
        else:
            return wrapper

interpreter = Interpreter()