import re
import inspect
import types
from zope.interface import implements
from funittest.core.interfaces import IParameterTest
from funittest.utils import normalizeClassName
from unittest import TestCase

class ParameterTest(TestCase):
    """A parameterized test can be run for different configurations. It keeps a
    registry of all configurations that have been provided.
    """

    implements(IParameterTest)

    parametertests = {}

    def __str__(self):
        format = ""
        p = re.compile('_[a-z]+')
        for name, value in inspect.getmembers(self):
            if re.match(p, name):
                if isinstance(value, types.StringTypes):
                    format += '%s="%s" ' % (name, getattr(self, name))
                else:
                    format += '%s=%s ' % (name, getattr(self, name))
        return "%s.test() with parameters %s" % (
            self.get_name(),
            format,
            )
 
    def configure(self, tests):
        # Register the parameter test
        self.parametertests = tests

    def get_name(self):
        """Return name of the test.
        """
        return normalizeClassName(self)
        
# Load test runner.
import funittest.runner.parametertestrunner