from funittest.runner.testrunner import SimpleTest
from zope.interface import implements
from funittest.core.interfaces import IPolicyTest
from funittest.problems import EnforceKeyProvided
from funittest.problems import PresetValueListExpected
from funittest.utils import normalizeClassName

import csv

class PolicyTest(SimpleTest):
    """A policy test can be run for different presets. It keeps a registry of
    all presets for which a policy should be tested according to a .csv file
    that contains the data and scenarios to be run.
    """

    implements(IPolicyTest)

    policytests = {}

    def provide(self, args):
        if not args.has_key('id'):
            raise EnforceKeyProvided(self, 'id')
        if not args.has_key('file'):
            raise EnforceKeyProvided(self, 'file')
        if not args.has_key('preset'):
            if self.fix.keys():
                raise EnforceKeyProvided(self, 'preset')
        else:
            for value in args["preset"].values():
                if type(value) == type(()) or type(value) == type([]):
                    pass
                else:
                    raise PresetValueListExpected(self, args)

        # Register the test and its presets
        preset = {}
        # Go through the schema to verify the dataproviders contain all
        # necessary fixed data
        if self.fix.keys():
            for dataprovider_id, dataprovider_keys in args["preset"].items():
                if not dataprovider_id in self.fix.keys():
                    msg = 'Fixed schema of policy test "%s" has no dataprovider registered under the id "%s".'
                    raise ValueError, msg  % (normalizeClassName(self), dataprovider_id)
                dp = self.fix[dataprovider_id]
                for dataprovider_key in dataprovider_keys:
                    if not dataprovider_key in dp:
                        msg = 'The dataprovider "%s" of the fixed schema "%s" does not provide the key "%s"'
                        raise ValueError, msg  % (normalizeClassName(dp), dataprovider_id, dataprovider_key)
                preset[dataprovider_id] = dataprovider_keys

        # Register the policy test
        self.policytests[args["id"]] = {"filename":args["file"], "preset":preset}

    def makeSuiteFromCSV(self, filename):
        """Load suite from given CSV.
        """
        csv_file = csv.reader(open(filename, 'r'))
        # First grab the header and parse out the variables
        header_fields = csv_file.next()
        header_fields = [ field.strip() for field in header_fields ]
        state_variables = [ field[1:] for field in header_fields if field.startswith('_') ]
        num_state_variables = len(state_variables)
        scenarios = [ field for field in header_fields if not field.startswith('_') ]
        num_scenarios = len(scenarios)
        
        tests = []
        # Now process the main bit of the CSV
        for bits in csv_file:
            bits = [ x.strip() for x in bits ]
            state = {}
            for x in range(num_state_variables):
                state[state_variables[x]] = bits[x]
    
            # Go through the scenarios and create the methods
            for x in range(num_scenarios):
                scenario = scenarios[x]
                yn = bits[num_state_variables + x]
                if yn == 'Y':
                    tests.append((scenario, state))
                elif yn =='N':
                    tests.append(('No'+scenario, state))
                else:
                    raise ValueError, 'Invalid response, must be Y/N: %s' % yn
        return state_variables, scenarios, tests


# Load test runner.
import funittest.runner.policytestrunner
