# Some boiler plate code parts for the SeleniumPlugin were taken from 
# django-sane-testing.:
#
# http://devel.almad.net/trac/django-sane-testing/
# 
# Copyright (c) 2008 Lukas "Almad" Linhart http://almad.net/
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the Lukas Linhart nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# THIS SOFTWARE IS PROVIDED BY Lukas Linhart ''AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import nose
from os.path import dirname
from os.path import basename
from os.path import abspath
from os.path import join
from nose import SkipTest
from nose.plugins import Plugin
from funittest import interpreter
from funittest.utils import SubProcess
from funittest.console.base import ModelBasedCommand
from funittest.console.runner import run_command
from funittest.seleniumrc import DEFAULTTIMEOUT
from funittest.log import _getLogPath
from funittest.log import logger
from configobj import ConfigObj

def enable_test(test_case, plugin_attribute):
    if not getattr(test_case, plugin_attribute, False):
        setattr(test_case, plugin_attribute, True)
    
def get_test_case_class(nose_test):
    if isinstance(nose_test.test, nose.case.MethodTestCase):
        return nose_test.test.test.im_class
    else:
        return nose_test.test.__class__        

class SeleniumPlugin(Plugin):
    """Start and stop browser sessions with Selenium"""

    activation_parameter = '--with-selenium'
    name = "selenium"

    def options(self, parser, env=os.environ):
        Plugin.options(self, parser, env)

    def configure(self, options, config):
        seleniumrc = ConfigObj("seleniumrc.ini")['seleniumrc']
        self._port = seleniumrc.get('port')
        self._host = seleniumrc.get('host')
        self._browserurl = seleniumrc.get('browserurl')
        self._browserstartcommand = seleniumrc.get('browserstartcommand')
        Plugin.configure(self, options, config)
       
    def startTest(self, test):
        script = sys.argv[0]
        bin_path = dirname(abspath(script))
        command = join(bin_path, 'seleniumrc')
        if not os.access(command, os.X_OK):
            selenium = None
        subprocess = SubProcess()        
        args = ['-timeout', str(DEFAULTTIMEOUT)]
        env = os.environ.copy()        

        log_name = _getLogPath(name=basename(command) + '.log')
        error_callback = None
        logger._logger.setLevel('CRITICAL')
        verbose = True
        subprocess.start(log_name, command, args, env, error_callback, verbose)
        test_case = get_test_case_class(test)
        enable_test(test_case, 'selenium_plugin_started')
        interpreter.setPort(self._port)
        interpreter.setHost(self._host)        
        interpreter.set_connection_state("ONLINE")
        interpreter.setBrowserURL(self._browserurl)
        interpreter.setBrowserStartCommand(self._browserstartcommand)
        interpreter.start()
        if getattr(test_case, "selenium_start", False):
            if isinstance(test.test, nose.case.MethodTestCase):
                test.test.test.im_self.selenium = interpreter
            else:
                raise SkipTest("I can only assign selenium to TestCase instance; argument passing will be implemented later") 
 
    def stopTest(self, test):
        test_case = get_test_case_class(test)
        if getattr(test_case, "selenium_start", False):
            # Stop Selenium
            test.test.test.im_self.selenium.stop()
            # Remove the trace of Selenium
            test.test.test.im_self.selenium = None
            



