from zope.interface import implements
from zope.component import adapts
from zope.component import provideAdapter
from funittest import objectEventNotify
from funittest import interpreter

from funittest.runner.testrunner import SimpleTestExecutor
from funittest.core.interfaces import IParameterTest
from funittest.core.interfaces import IParameterTestExecutor
from funittest.core.interfaces import ITestOptions
from funittest.core.events import TestStarting, TestEnding
from funittest.log import logger

class ParameterTestExecutor(SimpleTestExecutor):
    """Executor for parameter tests.
    """

    implements(IParameterTestExecutor)
    adapts(IParameterTest, ITestOptions)

    def run(self, result):
        """Run a parameter test.
        """
        if self.test.parametertests == None:
            pass # Desactivated        
        elif self.test.parametertests == {}:
            test_id = 'default'
            logger.run_parametertest(self.test, test_id=test_id, test_parameters={})
            self._run(result, test_id=test_id)
        else:            
            for test_id, test_parameters in self.test.parametertests.items():
                logger.run_parametertest(self.test, test_id, test_parameters)
                for key, value in test_parameters.items():
                    setattr(self.test, '_' + key, value)
                self._run(result, test_id)
        return result

    def _run(self, result, test_id='default'):
        self.start_test(self.test)
        objectEventNotify(TestStarting(self.test, test_id))
        self.execute(result)
        objectEventNotify(TestEnding(self.test))
        interpreter.pause_for_debugging(result.wasSuccessful())
        self.end_test()
    

    
provideAdapter(ParameterTestExecutor)
