from funittest import interpreter
from funittest import logger
from funittest.log import _getLogPath
from funittest.utils import FileAsPageTemplate

class PolicyTestResult:
    """
    Wrapper for test result
    """
    def __init__(self, result, policytest_id, policytest):
        self.result = result
        self.policytest_id = policytest_id
        self.policytest = policytest
        self.report = []

    def unwrap_result(self):
        return self.result

    def before_execute(self):
        # It is necessary to keep track of how many errors or failures
        # have occurred before running the test so that we can later
        # know what the result of the current test was.
        self.num_errors = len(self.result.errors)
        self.num_failures = len(self.result.failures)

    def after_execute(self, vary, scenario, scenario_names):
        was_successful = self.result.wasSuccessful()
        interpreter.pause_for_debugging(was_successful)
        
        # If there was an error, we need to keep track of it for the report
        def errorToStr(lst):
            errors = list()
            map(errors.extend, lst)
            return '\n'.join(map(str, errors))
        
        details = ""
        if len(self.result.errors) > self.num_errors:
            details += errorToStr(self.result.errors[self.num_errors:])
        if len(self.result.failures) > self.num_failures:
            details += errorToStr(self.result.failures[self.num_failures:])

        if details:
            details = '<pre>%s</pre>' % details

        opml_logger = logger.retrieveLogger('opml')
        if opml_logger:
            details += '<a href="file://%s">OPML Log</a>' % opml_logger.path()

        # Prepare the content of the report
        state = not ((len(self.result.errors) > self.num_errors) or 
                        len(self.result.failures) > self.num_failures)

        if scenario in scenario_names:
            yesno = 'Y'
        else:
            yesno = 'N'
        sce_name = (scenario in scenario_names) and scenario or scenario[2:]

        info = {sce_name: dict(yesno=yesno,
                                state=state,
                                details=details),
               }

        # FIXME: We need to merge same line for same vary
        info['vary'] = vary
        self.report.append(info)

    def output(self, fix, vary, scenario_names):
        """Write report in output file.
        """
        name = self.policytest.get_name()
        filename = _getLogPath(extras=(name,), name="%s.html" % self.policytest_id)
        options = dict(name=name,
                       presets=fix,
                       report=self.report,
                       variables=vary,
                       scenarios=scenario_names)
        template = FileAsPageTemplate('../templates/policy.pt')
        template(options=options, filename=filename)
