import os
import sys
import time
import traceback
from unittest import TestCase

from zope.interface import implements
from zope.component import adapts
from zope.component import provideAdapter
from funittest import objectEventNotify

from funittest import interpreter

from funittest.runner.baserunner import BaseTest
from funittest.runner.baserunner import TestInterpreterMixin
from funittest.core.interfaces import ITest
from funittest.core.interfaces import ITestExecutor
from funittest.core.interfaces import ITestOptions
from funittest.core.events import TestStarting
from funittest.core.events import TestEnding
from funittest.log import logger

class SimpleTestExecutor(TestInterpreterMixin):
    """A simple test executor.
    """

    def __init__(self, test, options):
        self.test = test
        self.options = options

    def execute(self, result):
        """Execute the given test.
        """

        if self.options.extension:
            self.test._testMethodName="test_" + self.options.extension
            self.test._testMethodDoc=""
            self.test._TestCase__testMethodName="test_" + self.options.extension
            self.test._TestCase__testMethodDoc=""
        else:
            self.test._testMethodName="test"
            self.test._testMethodDoc=""
            self.test._TestCase__testMethodName="test"
            self.test._TestCase__testMethodDoc=""

        # We should here set data to vary and fix

        startTime = time.time()

        logger.start_test(self.test)
        self.test.run(result)
        logger.end_test()

        stopTime = time.time()
        timeTaken = stopTime - startTime
    

class TestExecutor(SimpleTestExecutor):

    implements(ITestExecutor)
    adapts(ITest, ITestOptions)

    def run(self, result):
        """Default simple method used to call normal test:
        
        - start the interpreter;
        - run test;
        - pause if error;
        - stop the interpreter.
        """
        self.start_test(self.test)
        objectEventNotify(TestStarting(self.test))
        self.execute(result)
        objectEventNotify(TestEnding(self.test))
        interpreter.pause_for_debugging(result.wasSuccessful())
        self.end_test()


provideAdapter(TestExecutor)

class SimpleTest(BaseTest, TestCase):
    """
    Base class for tests
    
    A test can be executed with different configurations.
    """
    
    def __init__(self):
        self.configurations = []
    
    def run(self, result):
        TestCase.run(self, result)

class Test(SimpleTest):
    implements(ITest)

