# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""
Copyright 2006 ThoughtWorks, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
__docformat__ = "restructuredtext en"

# This file has been automatically generated via XSL

import os
import socket
import unittest
import time
import urllib
import httplib
import traceback
from funittest import interpreter
from funittest.baseinterpreter import BaseInterpreter
from selenium import selenium
from utils import FormatExceptionInfo
from funittest.log import logger
from funittest.problems import SeleniumResponseError
from funittest.core.assertions import TestCaseAssertions

NOTSTARTED = -1
DEFAULTTIMEOUT = 120000

class seleniumfixes(selenium):

    def do_command(self, verb, args):
        conn = self._get_http_connection(self.host, self.port)
        commandString = u'/selenium-server/driver/?cmd=' + urllib.quote_plus(unicode(verb).encode('utf-8'))
        for i in range(len(args)):
            commandString = commandString + '&' + unicode(i+1) + '=' + urllib.quote_plus(unicode(args[i]).encode('utf-8'))
        if (None != self.sessionId):
            commandString = commandString + "&sessionId=" + unicode(self.sessionId)
        conn.request("GET", commandString)
    
        response = conn.getresponse()
        data = unicode(response.read(), "UTF-8")
        reason = response.reason
        if (not data.startswith('OK')):
            logger.selenium_error(data, reason, commandString, traceback.format_stack())
            interpreter.pause_on_interpreter_error()
            raise SeleniumResponseError(data, reason, commandString)
        return data

    def get_boolean_array(self, verb, args):
        boolarr = self.get_string_array(verb, args)
        for i in range(len(boolarr)):
            if ("true" == boolarr[i]):
                boolarr[i] = True
                continue
            if ("false" == boolarr[i]):
                boolarr[i] = False
                continue
            raise ValueError, "result is neither 'true' nor 'false': " + boolarr[i]
        return boolarr

    def _get_http_connection(self, host, port):
        return httplib.HTTPConnection(self.host, self.port)

    def window_focus(self,windowName):
        """
        Gives focus to a window
        
        'windowName' is name of the window to be given focus
        """
        self.do_command("windowFocus", [windowName,])

    def window_maximize(self,windowName):
        """
        Resize window to take up the entire screen
        
        'windowName' is name of the window to be enlarged
        """
        self.do_command("windowMaximize", [windowName,])

    def get_log_messages(self):
        """
        Return the contents of the log.
        
        
        This is a placeholder intended to make the code generator make this API
        available to clients.  The selenium server will intercept this call, however,
        and return its recordkeeping of log messages since the last call to this API.
        Thus this code in JavaScript will never be called.
        
        The reason I opted for a servercentric solution is to be able to support
        multiple frames served from different domains, which would break a
        centralized JavaScript logging mechanism under some conditions.
        
        
        """
        return self.get_string("getLogMessages", [])

    def waitForVisible(self, locator):
        self.do_command("waitForVisible", [locator,])

    def set_context(self,context,logLevelThreshold):
        """
        Writes a message to the status bar and adds a note to the browser-side
        log.
        
        
        If logLevelThreshold is specified, set the threshold for logging
        to that level (debug, info, warn, error).
        
        (Note that the browser-side logs will \ *not* be sent back to the
        server, and are invisible to the Client Driver.)
        
        
        'context' is the message to be sent to the browser
        'logLevelThreshold' is one of "debug", "info", "warn", "error", sets the threshold for browser-side logging
        """
        self.do_command("setContext", [context,logLevelThreshold,])

    def _get_hrefs(self, locator, tag):
        # Get selected values
        js_call = "getIdsOfTagWithTypeByElement = function(element, tag) {" +\
                    "var elements = element.getElementsByTagName(tag);" +\
                    "var result = '';" +\
                    "for (var i = 0; i < elements.length; i++) {" +\
                    "    result += elements[i].href;" +\
                    "        result += ',';" +\
                    "}" +\
                    "return result;" +\
                    "};" +\
                    "getIdsOfTagWithType = function(locator, tag) { " +\
                    "var element = selenium.page().findElement(locator);" +\
                    "return getIdsOfTagWithTypeByElement(element, tag);" +\
                    "};" +\
                    "getIdsOfTagWithType(\"%s\",\"%s\")" % (locator, tag)
        hrefs = self.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        hrefs = hrefs.split(',')
        if hrefs:
            if hrefs[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                hrefs.pop()
        return hrefs

    def _getFormNames(self):
        js_call = "getFormNames = function() {" +\
                  "    var elements = selenium.page().getDocument().getElementsByTagName('form');" +\
                  "    var result = '';" +\
                  "    for (var i = 0; i < elements.length; i++) {" +\
                  "        result += elements[i].name;" +\
                  "        result += ',';" +\
                  "    };" +\
                  "    return result;"+\
                  "};" +\
                  "getFormNames();"
        names = self.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        names = names.split(',')
        if names:
            if names[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                names.pop()
        return names

    def _getFieldIds(self, form_locator):
        # Get selected values
        js_call = "getFieldIds = function(locator) {" +\
                  "    var element = selenium.page().findElement(locator);" +\
                  "    var elements = element.getElementsByTagName('div');" +\
                  "    var result = '';" +\
                  "    for (var i = 0; i < elements.length; i++) {" +\
                  "        if (elements[i].id != '') {" +\
                  "            result += elements[i].id;" +\
                  "            result += ',';" +\
                  "         };" +\
                  "     };" +\
                  "     return result;" +\
                  "};" +\
                  "getFieldIds(\"%s\");" % form_locator
        ids = self.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        ids = ids.split(',')
        if ids:
            if ids[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                ids.pop()
        return ids

    def _get_field_values(self, field_locator, input_type, input_name):
        # Get selected values
        js_call = "getValuesOfTagWithTypeByElement = function(element, tag, type,name) {" +\
                  "var elements = element.getElementsByTagName(tag);" +\
                  "var result = '';" +\
                  "for (var i = 0; i < elements.length; i++) {" +\
                  "    if (elements[i].type == type) {" +\
                  "        if (elements[i].name == name) {" +\
                  "            result += elements[i].value;" +\
                  "            result += ',';" +\
                  "        }" +\
                  "    }" +\
                  "}" +\
                  "return result;" +\
                  "};" +\
                  "getValuesOfTagWithType = function(locator, tag, type, name) { " +\
                  "var element = selenium.page().findElement(locator);" +\
                  "return getValuesOfTagWithTypeByElement(element, tag, type,name);" +\
                  "};" +\
                  "getValuesOfTagWithType(\"%s\",\"%s\",\"%s\",\"%s\")" % (field_locator, "input", input_type, input_name)
        values = self.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        values = values.split(',')
        if values:
            if values[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                values.pop()
        return values

    def _get_field_ids(self, field_locator, input_type):
        # Get selected values
        js_call = "getIdsOfTagWithTypeByElement = function(element, tag, type) {" +\
                  "var elements = element.getElementsByTagName(tag);" +\
                  "var result = '';" +\
                  "for (var i = 0; i < elements.length; i++) {" +\
                  "    if (elements[i].type == type) {" +\
                  "        result += elements[i].id;" +\
                  "        result += ',';" +\
                  "    }" +\
                  "}" +\
                  "return result;" +\
                  "};" +\
                  "getIdsOfTagWithType = function(locator, tag, type) { " +\
                  "var element = selenium.page().findElement(locator);" +\
                  "return getIdsOfTagWithTypeByElement(element, tag, type);" +\
                  "};" +\
                  "getIdsOfTagWithType(\"%s\",\"%s\",\"%s\")" % (field_locator, "input", input_type)
        ids = self.get_eval(js_call)
        # The ids are returned as a string with commas. Split the string.
        ids = ids.split(',')
        if ids:
            if ids[-1]=='':
                # The Javascript function always returns a list with a trailing comma
                ids.pop()
        return ids


class Shortcuts(object):

    def clickAndWait(self, locator, timeout=DEFAULTTIMEOUT):
        """
        Clicks and waits for page to load. 
        """
        self.do_command("click", [locator,])
        self.wait_for_page_to_load(timeout)

    def verifyTextPresent(self, pattern):
        if self.is_dry_run():
            return
        self.failUnless(self.is_text_present(pattern))

    def verifyTextNotPresent(self, pattern):
        if self.is_dry_run():
            return
        self.failIf(self.is_text_present(pattern))

    def verifyElementPresent(self, locator):
        if self.is_dry_run():
            return
        self.failUnless(self.is_element_present(locator))

    def verifyElementNotPresent(self, locator):
        if self.is_dry_run():
            return
        self.failIf(self.is_element_present(locator))

    def verifyVisible(self, locator):
        if self.is_dry_run():
            return
        self.failUnless(self.is_visible(locator))

    def verifyNotVisible(self, locator):
        if self.is_dry_run():
            return
        self.failIf(self.is_visible(locator))

class SeleniumRC(seleniumfixes, BaseInterpreter, TestCaseAssertions, Shortcuts):

    connection_state = "OFFLINE"

    def is_dry_run(self):
        return self.get_connection_state() == "OFFLINE"

    def set_connection_state(self, state):
        self.connection_state = state

    def get_connection_state(self):
        return self.connection_state

    def getBrowserURL(self):
        if self.browserURL is None:
            return ""
        else:
            return self.browserURL
        
    def getHost(self):
        return self.host

    def setHost(self, host):
        self.host = host

    def getPort(self):
        return self.port

    def setPort(self, port):
        self.port = port

    def getBrowserStartCommand(self):
        return self.browserStartCommand

    def setBrowserStartCommand(self, browserStartCommand):
        self.browserStartCommand = browserStartCommand

    def setBrowserURL(self, browserURL):
        self.browserURL = browserURL

    def do_command(self, verb, args):
        if verb in ['open', 'verifyLocation']:
            # Use complete urls
            if interpreter.getBrowserURL().endswith("/") and args[0].startswith("/"):
                args[0] = interpreter.getBrowserURL()[:-1] + args[0]
            elif not interpreter.getBrowserURL().endswith("/") and not args[0].startswith("/"):
                args[0] = interpreter.getBrowserURL() + "/" + args[0]
            else:
                args[0] = interpreter.getBrowserURL() + args[0]
        
        if self.is_dry_run():
            return

        try:
            result = self._do_command(verb, args)
        except socket.error:
            self.set_connection_state("OFFLINE")
            logger.exception()
            raise
        except:
            logger.exception()
            raise
        return result

    def _do_command(self, verb, args):
       return seleniumfixes.do_command(self, verb, args)

    def start(self):
        if self.is_dry_run():
            # Don't even try making a connection to Selenium Remote Control
            logger.selenium_start_offline()
            self.set_connection_state("OFFLINE")
            self.sessionId=None
        else:
            # Try establishing a connection to Selenium Remote Control
            logger.selenium_start_online(self.browserStartCommand, 
                                         self.browserURL)
            verb = "getNewBrowserSession"
            args = [self.browserStartCommand, self.browserURL]
            sessionId = None
            try:
                sessionId = self.get_string(verb, args, start=1)
            except socket.error:
                logger.exception()
            if sessionId == None:
                logger.selenium_start_offline()
                self.set_connection_state("OFFLINE")
                self.sessionId = None
            else:
                logger.selenium_session_established(self.sessionId)
                self.sessionId = sessionId
                self.set_connection_state("ONLINE")
                self.set_timeout(DEFAULTTIMEOUT)
    
    def get_string(self, verb, args, start=0):
        if start:
            #In the special case of starting, before we can know whether we have
            # to execute in dry run mode, or not, we need to find out about the 
            # state of the connection.
            pass
        else:
            # Usually, in dry run mode, just return an empty string, as there is
            # no real connection, and we don't need to make anything up.
            if self.is_dry_run():
                return ""
        try:
            result = self.do_command(verb, args)
        except socket.error:
            logger.exception()
            raise
        except:
            logger.exception()
            raise
            
        try:
            result = result.encode('utf-8')
        except:
            logger.exception()
            pass
            
        return str(result)[3:]
    
    def get_boolean(self, verb, args):
        if self.is_dry_run():
            return
        boolstr = self.get_string(verb, args)
        if ("true" == boolstr):
            return True
        if ("false" == boolstr):
            return False
        if ("e" == boolstr):
            # Error
            return False
        raise ValueError, "result is neither 'true' nor 'false' (or 'e' in case of an error): " + boolstr

    def pause(self, milliseconds):
        if self.is_dry_run():
            return
        time.sleep(milliseconds/1000)

